/*
 * $Id: DataGridDemo.java,v 1.66 2004/03/30 11:31:11 zfq Exp zfq $
 *
 * Copyright (C) 2001-2003 Extreme Component, Inc. All rights reserved.
 * Use is subject to license terms. 
 */

import java.awt.*;
import java.awt.event.*;
import java.awt.print.*;

import java.util.*;

import java.text.MessageFormat;
import java.text.DateFormat;

import java.net.URL;
import java.net.MalformedURLException;

import java.io.File;
import java.io.IOException;
import java.io.ObjectInputStream;
import java.io.ObjectOutputStream;
import java.io.FileInputStream;
import java.io.FileOutputStream;
import java.io.BufferedInputStream;
import java.io.BufferedOutputStream;

import java.beans.*;

import javax.swing.*;
import javax.swing.border.*;
import javax.swing.table.*;
import javax.swing.event.*;
import javax.swing.undo.*;
import javax.swing.filechooser.*;
import javax.swing.text.AttributeSet;
import javax.swing.text.MutableAttributeSet;
import javax.swing.text.SimpleAttributeSet;
import javax.swing.text.StyleConstants;

import javax.swing.plaf.*;
import javax.swing.plaf.metal.*;

import com.zfqjava.swing.JDataGrid;
import com.zfqjava.swing.JPrintPreview;
import com.zfqjava.swing.JStyleBar;
import com.zfqjava.swing.DataGridModel;
import com.zfqjava.swing.DefaultDataGridModel;
import com.zfqjava.swing.cell.Cell;
import com.zfqjava.swing.ModelIO;
import com.zfqjava.swing.DefaultFileFilter;
import com.zfqjava.swing.JTextFinder;
import com.zfqjava.swing.JTextSpeller;
import com.zfqjava.swing.JCommonPane;
import com.zfqjava.swing.PrintFactory;

/**
 * JDataGrid Demo
 *
 * @author $Author: zfq $
 * @version $Revision: 1.66 $ $Date: 2004/03/30 11:31:11 $
 */
public class DataGridDemo extends JPanel {

    // class fields
    private static final String NEW_COMMAND = "new";    
    private static final String OPEN_COMMAND = "open";
    private static final String SAVE_COMMAND = "save";
    private static final String SAVE_AS_COMMAND = "save-as";
    private static final String IMPORT_COMMAND = "import";
    private static final String EXPORT_COMMAND = "export";
    private static final String PAGE_SETUP_COMMAND = "page-setup";
    private static final String PRINT_COMMAND = "print";
    private static final String PRINT_PREVIEW_COMMAND = "print-preview";
    private static final String MRU_COMMAND = "mru";    
    private static final String EXIT_COMMAND = "exit";
    private static final String UNDO_COMMAND = "undo";
    private static final String REDO_COMMAND = "redo";
    private static final String CUT_COMMAND = "cut";
    private static final String COPY_COMMAND = "copy";
    private static final String PASTE_COMMAND = "paste";
    private static final String FIND_COMMAND = "find";
    private static final String FIND_NEXT_COMMAND = "find-next";
    private static final String REPLACE_COMMAND = "replace";
    private static final String GO_COMMAND = "go";    
    private static final String CLEAR_COMMAND = "clear";
    private static final String CLEAR_ALL_COMMAND = "clear-all";
    private static final String SELECT_ALL_COMMAND = "select-all";
    private static final String VIEW_TOOLBAR_COMMAND = "view-toolbar";
    private static final String VIEW_STYLEBAR_COMMAND = "view-stylebar";
    private static final String VIEW_ROWHEADER_COMMAND = "view-rowheader";
    private static final String VIEW_COLUMNHEADER_COMMAND = "view-columnheader";
    private static final String SET_IMAGE_COMMAND = "set-image";
    private static final String SPELL_COMMAND = "spell";        
    private static final String ROW_INSERT_BEFORE_COMMAND = "insert-row-before";
    private static final String ROW_INSERT_AFTER_COMMAND = "insert-row-after";        
    private static final String ROW_DELETE_COMMAND = "delete-row";
    private static final String COLUMN_INSERT_BEFORE_COMMAND = "insert-column-before";
    private static final String COLUMN_INSERT_AFTER_COMMAND = "insert-column-after";        
    private static final String COLUMN_DELETE_COMMAND = "delete-column";
    private static final String SET_ROW_COLUMN_COUNT_COMMAND = "set-row-column-count";    
    private static final String SET_COLUMN_NAME_COMMAND = "set-column-name";
    private static final String SET_COLUMN_CLASS_COMMAND = "set-column-class";
    private static final String SET_CELL_CLASS_COMMAND = "set-cell-class";
    private static final String SET_CELL_EDITABLE_COMMAND = "set-cell-editable";
    private static final String SET_CELL_ATTRIBUTES_COMMAND = "set-cell-attributes";
    private static final String SET_CELL_SPAN_COMMAND = "set-cell-span";
    private static final String SET_CELL_VALUE_COMMAND = "set-cell-value";
    private static final String MERGE_CELL_COMMAND = "merge-cell";
    private static final String SPLIT_CELL_COMMAND = "split-cell";
    private static final String SORT_ASCENDING_COMMAND = "sort-ascending";
    private static final String SORT_DESCENDING_COMMAND = "sort-descending";
    private static final String MOVE_CELL_COMMAND = "move-cell";
    private static final String CLEAR_CELL_COMMAND = "clear-cell";
    private static final String FILL_CELL_COMMAND = "fill-cell";
    private static final String TIP_COMMAND = "tip";    
    private static final String ABOUT_COMMAND = "about";
    private static final ResourceBundle bundle;
    private static final boolean DEBUG = false;
    
    static {
	bundle = ResourceBundle.getBundle("resources.datagriddemo");
    }

    // instance fields
    private JDataGrid dataGrid;
    private JMenuBar menuBar;
    private JToolBar toolBar;
    private JStyleBar styleBar;
    private JLabel statusBar;
    private JScrollPane scrollPane;
    private UndoManager undo;
    private PageFormat pageFormat;
    private File file;
    private Map actionMap;
    private JFrame f;
    private Handler handler;
    private JPopupMenu popupMenu;
    private java.util.List mruMenuItems;
    private java.util.List mruFiles;    
    private Map uiMap = new HashMap(2);
    private final int MRU_COUNT = 4;
    
    public DataGridDemo() {
	init();
	setLayout(new BorderLayout());
	add(createNorthPanel(), BorderLayout.NORTH);
	add(createCenterPanel(), BorderLayout.CENTER);
	add(statusBar, BorderLayout.SOUTH);
	addListeners(menuBar);
	addListeners(toolBar);
	addListeners(styleBar);	
    }
    public void updateUI() {
	super.updateUI();
	if(uiMap != null) {
	    for(Iterator i=uiMap.values().iterator(); i.hasNext(); ) {
		Component c = (Component)i.next();
		SwingUtilities.updateComponentTreeUI(c);
	    }
	}
    }
    
    private void init() {
	actionMap = new HashMap(30);
	handler = new Handler();
	undo = new UndoManager();		
	getAction(UNDO_COMMAND).setEnabled(false);
	getAction(REDO_COMMAND).setEnabled(false);
	mruFiles = new ArrayList(MRU_COUNT);
	mruMenuItems = new ArrayList(MRU_COUNT);
	// status bar
	statusBar = new JLabel();
	statusBar.setPreferredSize(new Dimension(100, 20));
	statusBar.setBorder(new EmptyBorder(1, 1, 1, 1));
	// end status bar
    }
    
    private JPanel createCenterPanel() {
	dataGrid = createDataGrid();
	scrollPane = new JScrollPane(dataGrid);
	JPanel panel = new JPanel(new BorderLayout());
	panel.add(scrollPane, BorderLayout.CENTER);
	styleBar.setDefaultAttributes(createAttributes(dataGrid));
	return panel;
    }

    // move to JDataGrid? JDataGrid.createDefaultCellAttributes
    private AttributeSet createAttributes(JDataGrid dataGrid) {
	SimpleAttributeSet a = new SimpleAttributeSet();
	Font font = dataGrid.getFont();
	Color fgColor = dataGrid.getForeground();
	Color bgColor = dataGrid.getBackground();
	StyleConstants.setFontFamily(a, font.getFamily());
	StyleConstants.setFontSize(a, font.getSize());	
	StyleConstants.setBold(a, font.isBold());
	StyleConstants.setItalic(a, font.isItalic());
	StyleConstants.setForeground(a, fgColor);
	StyleConstants.setBackground(a, bgColor);
	return a;
    }
    
    private JDataGrid createDataGrid() {
	JDataGrid dataGrid = new JDataGrid(createDataGridModel());
	// disable press key to start editing	
	dataGrid.putClientProperty("JTable.autoStartsEdit", Boolean.FALSE);
	// change the start click count to 2	
	dataGrid.putClientProperty("JDataGrid.clickCountToStartsEdit", new Integer(2));	
	dataGrid.setCellSelectionEnabled(true);
	dataGrid.setAutoResizeMode(JTable.AUTO_RESIZE_OFF);
	dataGrid.setRowHeight(0, 40);
	dataGrid.setRowHeight(4, 40);	
	dataGrid.getSelectionModel().addListSelectionListener(handler);
	dataGrid.getColumnModel().getSelectionModel().addListSelectionListener(handler);
	dataGrid.addUndoableEditListener(handler);
	dataGrid.addMouseListener(handler);
	return dataGrid;
    }

    private DataGridModel createDataGridModel() {
	DataGridModel datagridmodel = new DefaultDataGridModel(99, 26);
	Random random = new Random();
	datagridmodel.setValueAt("Date", 0, 0);
	datagridmodel.setCellEditable(false, 0, 0);
	for(int i=1; i<21; i++) {
	    datagridmodel.setCellClass(Date.class, i, 0);
	    datagridmodel.setValueAt(new java.util.Date(), i, 0);
	}
	datagridmodel.setValueAt("Number", 0, 1);
	datagridmodel.setCellEditable(false, 0, 1);	    
	for(int i=1; i<21; i++) {
	    datagridmodel.setCellClass(Number.class, i, 1);
	    datagridmodel.setValueAt(new Integer(random.nextInt()), i, 1);
	}
	datagridmodel.setValueAt("String", 0, 2);
	datagridmodel.setCellEditable(false, 0, 2);	    
	for(int i=1; i<21; i++) {
	    datagridmodel.setValueAt("hello, world", i, 2);
	}
	datagridmodel.setValueAt("Boolean", 0, 3);
	datagridmodel.setCellEditable(false, 0, 3);	    
	for(int i=1; i<21; i+=2) {
	    datagridmodel.setCellClass(Boolean.class, i, 3);
	    datagridmodel.setValueAt(Boolean.TRUE, i, 3);
	    datagridmodel.setCellClass(Boolean.class, i+1, 3);
	    datagridmodel.setValueAt(Boolean.FALSE, i+1, 3);		
	}
	datagridmodel.setValueAt("Mutli line String", 0, 4);
	datagridmodel.setCellEditable(false, 0, 4);	    
	datagridmodel.setCellClass(String.class, 0, 4);	    
	for(int i=1; i<21; i++) {
	    datagridmodel.setCellClass(String.class, i, 4);		
	    datagridmodel.setValueAt("hello, world", i, 4);
	}
	datagridmodel.setValueAt("Cell Span Demo", 0, 5);
	datagridmodel.setCellEditable(false, 0, 5);
	datagridmodel.merge(new Cell(0, 0, 5, 6));
	datagridmodel.merge(new Cell(1, 3, 5, 7));
	datagridmodel.merge(new Cell(5, 8, 5, 7));
	// demo attributes
	SimpleAttributeSet attr = new SimpleAttributeSet();
	// 0 row demo color
	attr = new SimpleAttributeSet();
	StyleConstants.setForeground(attr, Color.red);
	datagridmodel.setCellAttributes(attr, 0, 0);
	attr = new SimpleAttributeSet();	
	StyleConstants.setForeground(attr, Color.green);
	datagridmodel.setCellAttributes(attr, 0, 1);
	attr = new SimpleAttributeSet();	
	StyleConstants.setForeground(attr, Color.blue);
	datagridmodel.setCellAttributes(attr, 0, 2);
	attr = new SimpleAttributeSet();	
	StyleConstants.setForeground(attr, Color.yellow);
	datagridmodel.setCellAttributes(attr, 0, 3);
	attr = new SimpleAttributeSet();	
	StyleConstants.setForeground(attr, Color.pink);
	datagridmodel.setCellAttributes(attr, 0, 4);
	attr = new SimpleAttributeSet();	
	StyleConstants.setForeground(attr, Color.orange);
	datagridmodel.setCellAttributes(attr, 0, 5);	
	// 4 row demo font
	attr = new SimpleAttributeSet();
	StyleConstants.setBold(attr, true);
	datagridmodel.setCellAttributes(attr, 4, 0);
	attr = new SimpleAttributeSet();
	StyleConstants.setItalic(attr, true);
	datagridmodel.setCellAttributes(attr, 4, 1);
	attr = new SimpleAttributeSet();
	StyleConstants.setUnderline(attr, true);
	datagridmodel.setCellAttributes(attr, 4, 2);
	attr = new SimpleAttributeSet();
	StyleConstants.setStrikeThrough(attr, true);
	datagridmodel.setCellAttributes(attr, 4, 3);
	attr = new SimpleAttributeSet();
	StyleConstants.setFontFamily(attr, "Monospaced");
	datagridmodel.setCellAttributes(attr, 4, 4);
	attr = new SimpleAttributeSet();
	StyleConstants.setFontSize(attr, 22);
	datagridmodel.setCellAttributes(attr, 4, 5);
	datagridmodel.setValueAt("hello", 4, 5);
	// 8 row demo background color
	attr = new SimpleAttributeSet();
	StyleConstants.setBackground(attr, Color.red);
	datagridmodel.setCellAttributes(attr, 8, 0);
	attr = new SimpleAttributeSet();	
	StyleConstants.setBackground(attr, Color.green);
	datagridmodel.setCellAttributes(attr, 8, 1);
	attr = new SimpleAttributeSet();	
	StyleConstants.setBackground(attr, Color.blue);
	datagridmodel.setCellAttributes(attr, 8, 2);
	attr = new SimpleAttributeSet();	
	StyleConstants.setBackground(attr, Color.yellow);
	datagridmodel.setCellAttributes(attr, 8, 3);
	attr = new SimpleAttributeSet();	
	StyleConstants.setBackground(attr, Color.pink);
	datagridmodel.setCellAttributes(attr, 8, 4);
	attr = new SimpleAttributeSet();	
	StyleConstants.setBackground(attr, Color.orange);
	datagridmodel.setCellAttributes(attr, 8, 5);
	// 12 row demo icon
	attr = new SimpleAttributeSet();
	StyleConstants.setIcon(attr, getIcon("new.smicon"));
	datagridmodel.setCellAttributes(attr, 12, 0);
	attr = new SimpleAttributeSet();	
	StyleConstants.setIcon(attr, getIcon("open.smicon"));
	datagridmodel.setCellAttributes(attr, 12, 1);
	attr = new SimpleAttributeSet();	
	StyleConstants.setIcon(attr, getIcon("save.smicon"));
	datagridmodel.setCellAttributes(attr, 12, 2);
	attr = new SimpleAttributeSet();	
	StyleConstants.setIcon(attr, getIcon("save-as.smicon"));
	datagridmodel.setCellAttributes(attr, 12, 3);
	attr = new SimpleAttributeSet();
	StyleConstants.setIcon(attr, getIcon("import.smicon"));
	datagridmodel.setCellAttributes(attr, 12, 4);
	attr = new SimpleAttributeSet();	
	StyleConstants.setIcon(attr, getIcon("export.smicon"));
	datagridmodel.setCellAttributes(attr, 12, 5);	
	// demo background image
	attr = new SimpleAttributeSet();
	attr.addAttribute("background-image", getResource(getString("logo-image")));
	datagridmodel.setCellAttributes(attr, 5, 5);
	// demo format
	// 1-10 date format
	for(int i=1; i<11; i++) {
	    SimpleAttributeSet newAttr = new SimpleAttributeSet();
	    AttributeSet a = datagridmodel.getCellAttributes(i, 0);
	    if(a != null) {
		newAttr.addAttributes(a);
	    }
	    newAttr.addAttribute("format", DateFormat.getDateInstance());
	    datagridmodel.setCellAttributes(newAttr, i, 0);
	}
	// 11-20 time format
	for(int i=11; i<21; i++) {
	    SimpleAttributeSet newAttr = new SimpleAttributeSet();
	    AttributeSet a = datagridmodel.getCellAttributes(i, 0);
	    if(a != null) {
		newAttr.addAttributes(a);
	    }
	    newAttr.addAttribute("format", DateFormat.getTimeInstance());
	    datagridmodel.setCellAttributes(newAttr, i, 0);	    
	}
	return datagridmodel;
    }

    private JPanel createNorthPanel() {
	JPanel panel = new JPanel();
	panel.setLayout(new BorderLayout());
	panel.add(createMenuBar(), BorderLayout.NORTH);
	panel.add(createToolBar(), BorderLayout.CENTER);
	panel.add(createStyleBar(), BorderLayout.SOUTH);
	return panel;
    }
    
    private JMenuBar createMenuBar() {
	menuBar = new JMenuBar();
	menuBar.add(createFileMenu());
	menuBar.add(createEditMenu());
	menuBar.add(createViewMenu());
	menuBar.add(createToolMenu());
	menuBar.add(createLookAndFeelMenu());
	menuBar.add(createOrientationMenu());
	menuBar.add(createHelpMenu());
	return menuBar;
    }

    private JMenu createFileMenu() {
	JMenu menu = createMenu("file");
	menu.add(createMenuItem(NEW_COMMAND));
	menu.add(createMenuItem(OPEN_COMMAND));
	menu.add(createMenuItem(SAVE_COMMAND));
	menu.add(createMenuItem(SAVE_AS_COMMAND));
	menu.addSeparator();
	menu.add(createMenuItem(IMPORT_COMMAND));
	menu.add(createMenuItem(EXPORT_COMMAND));
	menu.addSeparator();	
	menu.add(createMenuItem(PAGE_SETUP_COMMAND));
	menu.add(createMenuItem(PRINT_COMMAND));
	menu.add(createMenuItem(PRINT_PREVIEW_COMMAND));	
	menu.addSeparator();
	menu.add(createMRUMenu());
	menu.addSeparator();	
	menu.add(createMenuItem(EXIT_COMMAND));				
	return menu;
    }
    private JMenu createMRUMenu() {
	JMenu menu = createMenu("recent-file");
	menu.setIcon(EmptyIcon.INSTANCE);
	addMRUMenuItem(menu);
	return menu;
    }
    private void addMRUMenuItem(JMenu menu) {
	for(int i=0; i<MRU_COUNT; i++) {
	    JMenuItem item = new JMenuItem();
	    item.setActionCommand(MRU_COMMAND);
	    item.addActionListener(handler);
	    item.setIcon(EmptyIcon.INSTANCE);
	    item.setVisible(false);
	    mruMenuItems.add(item);
	    menu.add(item);
	}
    }
    private void updateMRUMenuItems() {
	for(int i=0,n=mruMenuItems.size(); i<n; i++) {
	    JMenuItem item = (JMenuItem)mruMenuItems.get(i);
	    item.setText("");
	    item.setVisible(false);
	}
	for(int i=0,n=mruFiles.size(); i<n; i++) {
	    File f = (File)mruFiles.get(i);
	    JMenuItem item = (JMenuItem)mruMenuItems.get(i);
	    item.setText((i+1)+"."+f.getName());
	    item.setVisible(true);
	}
    }
    private JMenu createEditMenu() {
	JMenu menu = createMenu("edit");
	menu.add(createMenuItem(UNDO_COMMAND));
	menu.add(createMenuItem(REDO_COMMAND));
	menu.addSeparator();
	menu.add(createMenuItem(CUT_COMMAND));
	menu.add(createMenuItem(COPY_COMMAND));
	menu.add(createMenuItem(PASTE_COMMAND));
	menu.add(createMenuItem(CLEAR_COMMAND));
	menu.add(createMenuItem(CLEAR_ALL_COMMAND));	
	menu.addSeparator();	
	menu.add(createMenuItem(FIND_COMMAND));
	menu.add(createMenuItem(FIND_NEXT_COMMAND));
	menu.add(createMenuItem(REPLACE_COMMAND));
	menu.add(createMenuItem(GO_COMMAND));	
	menu.addSeparator();	
	menu.add(createMenuItem(SELECT_ALL_COMMAND));	
	return menu;
    }
    private JMenu createViewMenu() {
	JMenu menu = createMenu("view");
	menu.add(createCheckBoxMenuItem("Tool Bar", 0, handler, VIEW_TOOLBAR_COMMAND));
	menu.add(createCheckBoxMenuItem("Style Bar", 0, handler, VIEW_STYLEBAR_COMMAND));
	menu.add(createCheckBoxMenuItem("Row Header", 0, handler, VIEW_ROWHEADER_COMMAND));
	menu.add(createCheckBoxMenuItem("Column Header", 0, handler, VIEW_COLUMNHEADER_COMMAND));	
	return menu;
    }
    private JMenu createToolMenu() {
	JMenu menu = createMenu("tools");
	menu.add(createMenuItem(SPELL_COMMAND));	
	menu.add(createMenuItem(ROW_INSERT_BEFORE_COMMAND));
	menu.add(createMenuItem(ROW_INSERT_AFTER_COMMAND));
	menu.add(createMenuItem(ROW_DELETE_COMMAND));	
	menu.add(createMenuItem(COLUMN_INSERT_BEFORE_COMMAND));
	menu.add(createMenuItem(COLUMN_INSERT_AFTER_COMMAND));
	menu.add(createMenuItem(COLUMN_DELETE_COMMAND));
	menu.add(createMenuItem(MERGE_CELL_COMMAND));
	menu.add(createMenuItem(SPLIT_CELL_COMMAND));
	menu.add(createMenuItem(SORT_ASCENDING_COMMAND));
	menu.add(createMenuItem(SORT_DESCENDING_COMMAND));

  	menu.addSeparator();
//  	menu.add(createMenuItem(MOVE_CELL_COMMAND));
//  	menu.add(createMenuItem(FILL_CELL_COMMAND));
//  	menu.addSeparator();	
//  	menu.add(createMenuItem(SET_ROW_COLUMN_COUNT_COMMAND));
  	menu.add(createMenuItem(SET_COLUMN_NAME_COMMAND));
//  	menu.add(createMenuItem(SET_COLUMN_CLASS_COMMAND));
//  	menu.add(createMenuItem(SET_CELL_CLASS_COMMAND));
//  	menu.add(createMenuItem(SET_CELL_EDITABLE_COMMAND));
//  	menu.add(createMenuItem(SET_CELL_SPAN_COMMAND));
//  	menu.add(createMenuItem(SET_CELL_ATTRIBUTES_COMMAND));
//  	menu.add(createMenuItem(SET_CELL_VALUE_COMMAND));
//  	menu.add(createMenuItem(SET_IMAGE_COMMAND));	
	return menu;
    }

    private JMenu createLookAndFeelMenu() {
	JMenu menu = new JMenu("Look&Feel");
	menu.setMnemonic('L');
	UIManager.LookAndFeelInfo[] infos = UIManager.getInstalledLookAndFeels();
	ButtonGroup group = new ButtonGroup();
	String currentName = UIManager.getLookAndFeel().getName();
	ActionListener listener = new LookAndFeelActionListener();
	for(int i=0, count=infos.length; i<count; i++) {
	    JRadioButtonMenuItem rbItem = new JRadioButtonMenuItem(infos[i].getName());
	    if(currentName.equals(infos[i].getName())) {
		rbItem.setSelected(true);
	    }
	    rbItem.setActionCommand(infos[i].getClassName());
	    rbItem.addActionListener(listener);
	    group.add(rbItem);
	    menu.add(rbItem);
	    
	}
	return menu;
    }

    private class LookAndFeelActionListener implements ActionListener {
	public void actionPerformed(ActionEvent e) {
	    String command = e.getActionCommand();
	    try {
		UIManager.setLookAndFeel(command);
	    } catch (Exception exce) {
		JOptionPane.showMessageDialog(getFrame(), exce.getMessage());
		return;
	    }
	    SwingUtilities.updateComponentTreeUI(getFrame());
	    resizeWindow(getFrame());
	}
    }

    private JMenu createOrientationMenu() {
	JMenu menu = new JMenu("Orientation");
	menu.setMnemonic('O');
	ActionListener orientationListener = new OrientationActionListener();
	
	ButtonGroup group = new ButtonGroup();
	JRadioButtonMenuItem rbItem = new JRadioButtonMenuItem("Left To Right");
	rbItem.setMnemonic('L');
	rbItem.setActionCommand("lefttoright");
	rbItem.addActionListener(orientationListener);
	group.add(rbItem);
	menu.add(rbItem);

	rbItem =  new JRadioButtonMenuItem("Right To Left");
	rbItem.setMnemonic('R');	
	rbItem.setActionCommand("righttoleft");
	rbItem.addActionListener(orientationListener);	
	group.add(rbItem);
	menu.add(rbItem);

	rbItem =  new JRadioButtonMenuItem("Unknown");
	rbItem.setMnemonic('U');	
	rbItem.setSelected(true);
	rbItem.setActionCommand("unknown");
	rbItem.addActionListener(orientationListener);	
	group.add(rbItem);
	menu.add(rbItem);
	
	return menu;
    }

    private class OrientationActionListener implements ActionListener {

	public void actionPerformed(ActionEvent e) {
	    String command = e.getActionCommand();
	    Window w = getFrame();
	    ComponentOrientation orientation = null;
	    if(command.equals("lefttoright")) {
		orientation = ComponentOrientation.LEFT_TO_RIGHT;
	    } else if(command.equals("righttoleft")) {
		orientation = ComponentOrientation.RIGHT_TO_LEFT;
	    } else if(command.equals("unknown")) {
		orientation = ComponentOrientation.UNKNOWN;
	    }
	    setComponentTree(w, orientation);
	    resizeWindow(w);	    
	}
						    
    }

    private void setComponentTree(Component c, ComponentOrientation orientation) {
	if(c instanceof Container) {
	    Component[] components = ((Container)c).getComponents();
	    for(int i=0; i<components.length; i++) {
		components[i].setComponentOrientation(orientation);
		components[i].invalidate();
		setComponentTree(components[i], orientation);
	    }
	}
    }
    
    private JMenu createHelpMenu() {
	JMenu menu = createMenu("help");
	menu.add(createMenuItem(TIP_COMMAND));
	menu.add(createMenuItem(ABOUT_COMMAND));
	return menu;
    }

    private JMenuItem createCheckBoxMenuItem(String name, int mnemonic, ActionListener listener, String command) {
	JMenuItem item = new JCheckBoxMenuItem(name);
	item.setMnemonic(mnemonic);
	item.setSelected(true);
	item.addActionListener(listener);
	item.setActionCommand(command);
	return item;
    }    
    
    private JToolBar createToolBar() {
	toolBar = new JToolBar();
	toolBar.setFloatable(false);	
	toolBar.putClientProperty("JToolBar.isRollover", Boolean.TRUE);	
	toolBar.add(createButton(NEW_COMMAND));
	toolBar.add(createButton(OPEN_COMMAND));
	toolBar.add(createButton(SAVE_COMMAND));
	toolBar.addSeparator();
	toolBar.add(createButton(PRINT_COMMAND));
	toolBar.add(createButton(PRINT_PREVIEW_COMMAND));
	toolBar.addSeparator();	
	toolBar.add(createButton(UNDO_COMMAND));	
	toolBar.add(createButton(REDO_COMMAND));
	toolBar.addSeparator();
	toolBar.add(createButton(CUT_COMMAND));
	toolBar.add(createButton(COPY_COMMAND));	
	toolBar.add(createButton(PASTE_COMMAND));	
	toolBar.addSeparator();		
	toolBar.add(createButton(ROW_INSERT_BEFORE_COMMAND));
	toolBar.add(createButton(ROW_INSERT_AFTER_COMMAND));
	toolBar.add(createButton(ROW_DELETE_COMMAND));	
	toolBar.add(createButton(COLUMN_INSERT_BEFORE_COMMAND));
	toolBar.add(createButton(COLUMN_INSERT_AFTER_COMMAND));
	toolBar.add(createButton(COLUMN_DELETE_COMMAND));
	toolBar.add(createButton(MERGE_CELL_COMMAND));
	toolBar.add(createButton(SPLIT_CELL_COMMAND));
	toolBar.add(createButton(SORT_ASCENDING_COMMAND));
	toolBar.add(createButton(SORT_DESCENDING_COMMAND));		
	return toolBar;
    }
	
    private JStyleBar createStyleBar() {
	styleBar = new JStyleBar();
	styleBar.setFloatable(false);	
	styleBar.addChangeListener(handler);
	return styleBar;
    }
    
    private void doNew() {
	dataGrid.setModel(new DefaultDataGridModel(99, 26));
	setFile(null);
    }
    private String formatOpenMsg(File f) {
	String msg = f.getName();
	return MessageFormat.format(getString("openfile.msg"), new String[] { msg });
    }
    private void doOpen(File f) {
	if(f == null) return;
	ObjectInputStream ios = null;	
	try {
	    ios = new ObjectInputStream(new BufferedInputStream(new ProgressMonitorInputStream(getFrame(), formatOpenMsg(f), new FileInputStream(f))));
	    DefaultDataGridModel model = (DefaultDataGridModel)ios.readObject();
	    dataGrid.setModel(model);
	    setFile(f);
	} catch (Exception ex) {
	    JOptionPane.showMessageDialog(getFrame(), ex);
	} finally {
	    try {
		ios.close();
	    } catch (Exception e) {
		e.printStackTrace();
	    }
	}
    }
    private void doOpen() {
	JFileChooser fc = createFileChooser();
	int retValue = fc.showOpenDialog(getFrame());
	if(retValue == JFileChooser.APPROVE_OPTION) {
	    File f = fc.getSelectedFile();
	    doOpen(f);
	}	
    }
    private void doSave(File f) {
	if(f == null) return;
	ObjectOutputStream oos = null;	
	try {
	    oos = new ObjectOutputStream(new BufferedOutputStream(new FileOutputStream(f)));
	    oos.writeObject(dataGrid.getModel());
	    oos.flush();
	    oos.close();	
	    resetUndoManager();		
	} catch (Exception ex) {
	    JOptionPane.showMessageDialog(getFrame(), ex);
	} finally {
	    try {
		oos.close();
	    } catch (Exception e) {
		e.printStackTrace();
	    }
	}
    }
    private void doSave() {
	if(file == null) {
	    doSaveAs();
	} else {
	    doSave(file);
	}
    }
    private void doSaveAs() {
	JFileChooser fc = createFileChooser();
	int retValue = fc.showSaveDialog(getFrame());
	if(retValue == JFileChooser.APPROVE_OPTION) {
	    File f = fc.getSelectedFile();
	    doSave(f);
	    setFile(f);
	}	
    }
    private void doImport() {
	JFileChooser fc = createFileChooserForImport();
	int retValue = fc.showDialog(getFrame(), null);
	if(retValue == JFileChooser.APPROVE_OPTION) {
	    try {
		File f = fc.getSelectedFile();
		if(f != null) {
		    String format = getExtension(f);
		    TableModel model = ModelIO.readTableModel(f, format, null);
		    if(model != null) {
			dataGrid.setModel(model);
		    }
		}
	    } catch (Exception ex) {
		JOptionPane.showMessageDialog(getFrame(), ex);
		ex.printStackTrace();
	    }
	}	
    }
    private void doExport() {
	TableModel model = dataGrid.getModel();
	JFileChooser fc = createFileChooserForExport();
	int retValue = fc.showDialog(getFrame(), null);
	if(retValue == JFileChooser.APPROVE_OPTION) {
	    try {
		File f = fc.getSelectedFile();
		if(f != null) {
		    String format = getExtension(f);		
		    ModelIO.writeTableModel(model, format, null, f);
		}
	    } catch (Exception ex) {
		JOptionPane.showMessageDialog(getFrame(), ex);
	    }
	}	
    }    
    private void doPageSetup() {
	PrinterJob job = PrinterJob.getPrinterJob();
	pageFormat = job.pageDialog(job.defaultPage());
    }
    private void doPrint() {
	PrinterJob job = PrinterJob.getPrinterJob();
	job.setPrintable(createDataGridPrintable());
	job.printDialog();
    }
    private void doPrintPreview() {
	JPrintPreview pre = new JPrintPreview() {
		public void close() {
		    getFrame().getContentPane().removeAll();
		    getFrame().getContentPane().add(DataGridDemo.this);
		    getFrame().validate();
		    getFrame().repaint(); 
		}
	    };
	if(pageFormat != null) {
	    pre.setPageFormat(pageFormat);
	}
	getFrame().getContentPane().removeAll();
	getFrame().getContentPane().add(pre);
	getFrame().validate();
	getFrame().repaint(); 
	// should move to JPrintPreview, use flag to control asynchronized?
	pre.setPrintData(createDataGridPrintable());
	/*
	  getFrame().hide();	  
	  pre.showDialog(getFrame());
	  getFrame().show();
	  getFrame().toFront();
	*/
    }
    private void doMRU(int index) {
	if(index >=0 && index < mruFiles.size()) {
	    File f = (File)mruFiles.get(index);
	    doOpen(f);
	}
    }
    private void doExit() {
	int option = JOptionPane.showConfirmDialog(getFrame(), "Do you save model?");
	if(option == JOptionPane.YES_OPTION) {
	    doSave();
	    System.exit(0);
	} else if(option == JOptionPane.NO_OPTION) {
	    System.exit(0);
	} else {
	    // do nothing
	}
    }
    private void doSplash() {
    }
    private void doTip() {
	showTipOfTheDayDialog(getFrame());
    }
    private void doAbout() {
	Map map = new HashMap();
	map.put(JCommonPane.PRODUCT_NAME, getString("product-name"));
	map.put(JCommonPane.COPYRIGHT_NOTE, getString("copyright-note"));
	JCommonPane.showAboutDialog(getFrame(), map);
    }
    private void doUndo() {
	try {
	    undo.undo();
	} catch (CannotUndoException ex) {
	    ex.printStackTrace();		
	}
	updateUndoActions();	
    }
    private void doRedo() {
	try {
	    undo.redo();
	} catch (CannotUndoException ex) {
	    ex.printStackTrace();		
	}
	updateUndoActions();	
    }
    private void doFind() {
	JTextFinder finder = createTextFinder();
	finder.setDialogType(JTextFinder.FIND_DIALOG);	
	finder.showDialog(getFrame());
    }
    private void doFindNext() {
	JTextFinder finder = createTextFinder();
	finder.findNext();
    }
    private void doReplace() {
	JTextFinder finder = createTextFinder();
	finder.setDialogType(JTextFinder.REPLACE_DIALOG);
	finder.showDialog(getFrame());	
    }
    private void doGo() {
	String title = "Go to cell";
	String rowLabel = "Row:";
	JTextField rowField = new JTextField(9);	
	String columnLabel = "Column:";
	JTextField columnField = new JTextField(9);
	Object[] message = new Object[4];
	message[0] = rowLabel;
	message[1] = rowField;
	message[2] = columnLabel;
	message[3] = columnField;
	int retValue = JOptionPane.showOptionDialog(getFrame(),
						    message,
						    title,
						    JOptionPane.OK_CANCEL_OPTION,
						    JOptionPane.INFORMATION_MESSAGE,
						    null,
						    null,
						    null);
	if(retValue == JOptionPane.OK_OPTION) {
	    String rowText = rowField.getText();
	    String columnText = columnField.getText();
	    try {
		int row = Integer.parseInt(rowText);
		int column = Integer.parseInt(columnText);
		row--;
		column--;
		if(row < 0 || row > dataGrid.getRowCount()-1 ||
		   column < 0 || column > dataGrid.getColumnCount() -1) {
		    return;
		}
		dataGrid.changeSelection(row, column, false, false);
	    } catch (NumberFormatException e) {
		JOptionPane.showMessageDialog(getFrame(), "Incorrect cell number");
	    }
	}
    }
    private void doClear() {
	Cell c = dataGrid.getSelectedCell();	
	if(c != null) {
	    clearCell(c);
	}	    
    }
    private void doClearAll() {
	int rows = dataGrid.getRowCount();
	int cols = dataGrid.getColumnCount();
	if(rows > 0 && cols > 0) {
	    Cell c = new Cell(0, rows-1, 0, cols-1);
	    clearCell(c);
	}
    }
    private void clearCell(Cell c) {
	DefaultDataGridModel model = getModel();
	if(model != null) {
	    if(!model.clear(c)) {
		JOptionPane.showMessageDialog(getFrame(), "The cell has been cleared and need not clear or \nCannot clear selected  cells");
	    }
	}
    }
    private void doSelectAll() {
	dataGrid.selectAll();
    }
    private void doViewToolBar(boolean b) {
	toolBar.setVisible(b);
    }
    private void doViewStyleBar(boolean b) {
	styleBar.setVisible(b);
    }
    private void doViewRowHeader(boolean b) {
	Component header = scrollPane.getRowHeader();
	if(header != null) {
	    header.setVisible(b);
	}
    }
    private void doViewColumnHeader(boolean b) {
	Component header = scrollPane.getColumnHeader();
	if(header != null) {
	    header.setVisible(b);
	}
    }
    private void doSpell() {
	JTextSpeller speller = createTextSpeller();
	speller.showDialog(getFrame());
    }
    private void doSetImage() {
	Cell c = dataGrid.getSelectedCell();	
	if(c != null) {
	    JFileChooser fc = createFileChooserForImage();
	    int retValue = fc.showOpenDialog(getFrame());
	    if(retValue == JFileChooser.APPROVE_OPTION) {
		File f = fc.getSelectedFile();
		if(f != null && isImage(f)) {
		    try {
			URL url = f.toURL();
			setBackgroundImage(c.getFirstRow(), c.getFirstColumn(), url);
		    } catch (MalformedURLException e) {
			e.printStackTrace();
		    }
		}
	    }
	}
    }
    private void doRowInsert(boolean before) {
	DefaultDataGridModel model = getModel();
	if(model != null) {
	    int row = dataGrid.getSelectedRow();
	    if(row != -1) {
		if(before) {
		    model.addRow(row, null);
		} else {
		    model.addRow(row+1, null);
		}
	    }
	}
    }
    private void doRowDelete() {
	DefaultDataGridModel model = getModel();
	if(model != null) {
	    int row = dataGrid.getSelectedRow();
	    if(row != -1) {
		model.removeRow(row);
	    }	    
	}	
    }
    private void doColumnInsert(boolean before) {
	DefaultDataGridModel model = getModel();
	if(model != null) {
	    int col = dataGrid.getSelectedColumn();
	    if(col != -1) {
		if(before) {
		    model.addColumn(col, null);
		} else {
		    model.addColumn(col+1, null);
		}
	    }
	}	
    }
    private void doColumnDelete() {
	DefaultDataGridModel model = getModel();
	if(model != null) {
	    int col = dataGrid.getSelectedColumn();
	    if(col != -1) {
		model.removeColumn(col);
	    }
	}	
    }
    private void doCellMerge() {
	Cell c = dataGrid.getSelectedCell();
	DefaultDataGridModel model = getModel();
	if(c != null && model != null) {
	    if(!model.merge(c)) {
		JOptionPane.showMessageDialog(getFrame(), "The cell has been merged or\nCannot merge selected cells");
	    }
	}
    }
    private void doCellSplit() {
	Cell c = dataGrid.getSelectedCell();
	DefaultDataGridModel model = getModel();
	if(c != null && model != null) {
	    if(!model.split(c)) {
		JOptionPane.showMessageDialog(getFrame(), "The cell has been splited or\nCannot split selected cells");
	    }
	}	
    }
    private void doSort(boolean ascending) {
	Cell c = dataGrid.getSelectedCell();	
	DefaultDataGridModel model = getModel();
	if(c != null && model != null) {
	    if(!model.sort(c, ascending)) {
		JOptionPane.showMessageDialog(getFrame(), "The cell has been sorted or\nCannot sort selected  cells:\nOnly sort single row or single column");
	    }
	}	    
    }
    private void doSetColumnName() {
	int column = dataGrid.getSelectedColumn();
	DefaultDataGridModel model = getModel();
	if(column >= 0 && model != null) {
	    String name = JOptionPane.showInputDialog(getFrame(), "Please input the column name");
	    if(name != null) {
		model.setColumnName(name, column);
	    }
	}
    }
    private void doMove() {
	Cell c = dataGrid.getSelectedCell();	
	DefaultDataGridModel model = getModel();
	if(c != null && model != null) {
	    /* Todo:
	       JOptionPane.showMessageDialog(getFrame(), "Cannot move selected  cells");
	    */
	}	    
    }
    private void doFill() {
	Cell c = dataGrid.getSelectedCell();	
	DefaultDataGridModel model = getModel();
	if(c != null && model != null) {
	    /* Todo:
	       JOptionPane.showMessageDialog(getFrame(), "The cell has been filled and need not fill again or\nCannot fill selected  cells");
	    */
	}	    
    }
    private void doCellAttributes() {
	Cell c = dataGrid.getSelectedCell();
	DefaultDataGridModel model = getModel();	
	if(c != null && model != null) {
	    int row = c.getFirstRow();
	    int col = c.getFirstColumn();
	    AttributeSet a = model.getCellAttributes(row, col);
	}
    }
    //
    // utility methods
    //
    private JTextFinder createTextFinder() {
	JTextFinder finder = (JTextFinder)uiMap.get("finder");
	if(finder == null) {
	    finder = new JTextFinder();
	    finder.setForComponent(dataGrid);
	    uiMap.put("finder", finder);
	}
	return finder;
    }
    private JTextSpeller createTextSpeller() {
	JTextSpeller speller = (JTextSpeller)uiMap.get("speller");
	if(speller == null) {
	    speller = new JTextSpeller();
	    speller.setForComponent(dataGrid);
	    uiMap.put("speller", speller);
	}
	return speller;
    }    
    private JFileChooser createFileChooser() {
	JFileChooser fc = new JFileChooser(new File(System.getProperty("user.dir")));
	fc.addChoosableFileFilter(new DefaultFileFilter("ser", getString("ser.desc")));
	return fc;
    }
    private JFileChooser createFileChooserForImage() {
	JFileChooser fc = new JFileChooser(new File(System.getProperty("user.dir")));
	fc.addChoosableFileFilter(new DefaultFileFilter(new String[] {"gif", "jpg", "jpeg", "png"}, getString("image.desc")));
	return fc;
    }    
    private JFileChooser createFileChooserForImport() {
	JFileChooser fc = new JFileChooser(new File(System.getProperty("user.dir")));
	fc.setApproveButtonText(getString("importButton.name"));
	Integer mnemonic = getMnemonic("importButton.mnemonic");
	if(mnemonic != null) {
	    fc.setApproveButtonMnemonic(mnemonic.intValue());
	}
	fc.setApproveButtonToolTipText(getString("importButton.desc"));	
	fc.addChoosableFileFilter(new DefaultFileFilter("txt", getString("txt.desc")));
	fc.addChoosableFileFilter(new DefaultFileFilter("csv", getString("csv.desc")));
	fc.addChoosableFileFilter(new DefaultFileFilter(new String[] {"html","htm"}, getString("html.desc")));
	fc.setAcceptAllFileFilterUsed(false);
	return fc;
    }
    private JFileChooser createFileChooserForExport() {
	JFileChooser fc = new JFileChooser(new File(System.getProperty("user.dir")));
	fc.setApproveButtonText(getString("exportButton.name"));
	Integer mnemonic = getMnemonic("exportButton.mnemonic");
	if(mnemonic != null) {
	    fc.setApproveButtonMnemonic(mnemonic.intValue());
	}	
	fc.setApproveButtonToolTipText(getString("exportButton.desc"));
	fc.addChoosableFileFilter(new DefaultFileFilter("txt", getString("txt.desc")));
	fc.addChoosableFileFilter(new DefaultFileFilter("csv", getString("csv.desc")));
	fc.addChoosableFileFilter(new DefaultFileFilter(new String[] {"html","htm"}, getString("html.desc")));
	fc.setAcceptAllFileFilterUsed(false);	
	return fc;
    }    
    private static String getExtension(File f) {
	String name = f.getName();
	int index = name.lastIndexOf(".");
	if(index != -1) {
	    name = name.substring(index+1);
	}
	return name;
    }
    private boolean isImage(File f) {
	String name = f.getName();
	int index = name.lastIndexOf(".");
	if(index != -1) {
	    name = name.substring(index+1);
	    if(name != null) {
		if(name.equalsIgnoreCase("gif") ||
		   name.equalsIgnoreCase("jpg") ||
		   name.equalsIgnoreCase("jpeg") ||
		   name.equalsIgnoreCase("png")) {
		    return true;
		}
	    }
	}
	return false;
    }
    private void setFile(File file) {
	this.file = file;
	resetUndoManager();
	getFrame().setTitle(getTitle());
	if(file != null) {
	    if(mruFiles.contains(file)) {
		mruFiles.remove(file);
		mruFiles.add(0, file);
	    } else {
		mruFiles.add(0, file);
		if(mruFiles.size() > MRU_COUNT) {
		    mruFiles.remove(mruFiles.size()-1);
		}
	    }
	    updateMRUMenuItems();
	}
    }
    private String getTitle() {
	if(file == null) {
	    return "Untitled" + " - " + getString("title");
	} else {
	    return file.getName() + " - " + getString("title");
	}
    }
    private void resetUndoManager() {
	undo.discardAllEdits();
	updateUndoActions();	
    }
    private void updateUndoActions() {
	getAction(UNDO_COMMAND).setEnabled(undo.canUndo());
	getAction(REDO_COMMAND).setEnabled(undo.canRedo());
    }
    private void open(File f) throws Exception {
    }
    private void save(File f, TableModel model) throws Exception {
    }
    private Printable createDataGridPrintable() {
	Map map = new HashMap();
	map.put(PrintFactory.HORIZONTAL_ALIGNMENT, new Integer(SwingConstants.CENTER));
	// change the print orientation, default is SwingConstants.HORIZONTAL
	// map.put(PrintFactory.ORIENTATION, new Integer(SwingConstants.VERTICAL));
	map.put(PrintFactory.HEADER_FORMAT, new MessageFormat(getTitle()));
	map.put(PrintFactory.HEADER_MARGIN, new Insets(0, 0, 6, 0));
	map.put(PrintFactory.HEADER_ALIGNMENT, new Integer(SwingConstants.CENTER));	
	map.put(PrintFactory.FOOTER_FORMAT, new MessageFormat("Page {0}"));
	map.put(PrintFactory.FOOTER_MARGIN, new Insets(6, 0, 0, 0));
	map.put(PrintFactory.FOOTER_ALIGNMENT, new Integer(SwingConstants.RIGHT));	
	return PrintFactory.createPrintable(dataGrid, map);
    }
    private DefaultDataGridModel getModel() {
	TableModel model = dataGrid.getModel();
	if(model instanceof DefaultDataGridModel) {
	    return (DefaultDataGridModel)model;
	}
	return null;
    }
    private void setBackgroundImage(int row, int col, URL url) {
	DefaultDataGridModel model = getModel();
	if(model != null) {	
	    AttributeSet a = model.getCellAttributes(row, col);
	    if(a instanceof MutableAttributeSet) {
		((MutableAttributeSet)a).addAttribute("background-image", url);
		model.setCellAttributes(a, row, col);		
	    } else {
		SimpleAttributeSet newa = new SimpleAttributeSet();
		if(a != null) {
		    newa.addAttributes(a);
		}
		newa.addAttribute("background-image", url);
		model.setCellAttributes(newa, row, col);
	    }
	}
    }
    private JFrame getFrame() {
	if(f == null) {
	    f = new JFrame(getTitle());
	    f.addWindowListener(handler);
	    f.setDefaultCloseOperation(WindowConstants.DO_NOTHING_ON_CLOSE);
	    f.getContentPane().add(this);
	}
	return f;
    }
    

    // inner class
    private class Handler implements UndoableEditListener,
				     ListSelectionListener,
				     ChangeListener,
				     ActionListener,
				     WindowListener,
				     MouseListener {
	public void undoableEditHappened(UndoableEditEvent e) {
	    UndoableEdit edit = e.getEdit();
	    undo.addEdit(edit);
	    updateUndoActions();
	    if(DEBUG) {
		System.out.println(edit);
	    }
	}
	public void valueChanged(ListSelectionEvent e) {
	    if(e.getValueIsAdjusting()) return;
	    TableModel model = dataGrid.getModel();
	    if(model instanceof DataGridModel) {
		int row = dataGrid.getSelectedRow();
		int col = dataGrid.getSelectedColumn();
		if(row != -1 && col != -1) {
		    styleBar.setAttributes(((DataGridModel)model).getCellAttributes(row, col));
		    if(DEBUG) {
			System.out.println("Cell Attributes:" + ((DataGridModel)model).getCellAttributes(row, col));
		    }
		}
	    }
	}
	public void stateChanged(ChangeEvent e) {
	    TableModel model = dataGrid.getModel();
	    if(model instanceof DataGridModel) {
		int row = dataGrid.getSelectedRow();
		int col = dataGrid.getSelectedColumn();
		if(row != -1 && col != -1) {
		    ((DataGridModel)model).setCellAttributes(styleBar.getAttributes(), row, col);
		}
	    }
	    if(DEBUG) {
		System.out.println(styleBar.getAttributes());
	    }
	}
	public void actionPerformed(ActionEvent e) {
	    String command = e.getActionCommand();
	    Object source = e.getSource();
	    if(command == NEW_COMMAND) {
		doNew();
	    } else if(command == OPEN_COMMAND) {
		doOpen();
	    } else if(command == SAVE_COMMAND) {
		doSave();
	    } else if(command == SAVE_AS_COMMAND) {
		doSaveAs();
	    } else if(command == IMPORT_COMMAND) {
		doImport();
	    } else if(command == EXPORT_COMMAND) {
		doExport();
	    } else if(command == PAGE_SETUP_COMMAND) {
		doPageSetup();
	    } else if(command == PRINT_COMMAND) {
		doPrint();
	    } else if(command == PRINT_PREVIEW_COMMAND) {
		doPrintPreview();
	    } else if(command == MRU_COMMAND) {
		doMRU(mruMenuItems.indexOf(source));
	    } else if(command == EXIT_COMMAND) {
		doExit();
	    } else if(command == UNDO_COMMAND) {
		doUndo();
	    } else if(command == REDO_COMMAND) {
		doRedo();
	    } else if(command == FIND_COMMAND) {
		doFind();
	    } else if(command == FIND_NEXT_COMMAND) {
		doFindNext();
	    } else if(command == REPLACE_COMMAND) {
		doReplace();
	    } else if(command == GO_COMMAND) {
		doGo();
	    } else if(command == CLEAR_COMMAND) {
		doClear();
	    } else if(command == CLEAR_ALL_COMMAND) {
		doClearAll();
	    } else if(command == SELECT_ALL_COMMAND) {
		doSelectAll();
	    } else if(command == VIEW_TOOLBAR_COMMAND) {
		doViewToolBar(((JCheckBoxMenuItem)source).isSelected());
	    } else if(command == VIEW_STYLEBAR_COMMAND) {
		doViewStyleBar(((JCheckBoxMenuItem)source).isSelected());
	    } else if(command == VIEW_ROWHEADER_COMMAND) {
		doViewRowHeader(((JCheckBoxMenuItem)source).isSelected());
	    } else if(command == VIEW_COLUMNHEADER_COMMAND) {
		doViewColumnHeader(((JCheckBoxMenuItem)source).isSelected());
	    } else if(command == SET_IMAGE_COMMAND) {
		doSetImage();
	    } else if(command == SET_CELL_ATTRIBUTES_COMMAND) {
		doCellAttributes();
	    } else if(command == SPELL_COMMAND) {
		doSpell();
	    } else if(command == ROW_INSERT_BEFORE_COMMAND) {
		doRowInsert(true);
	    } else if(command == ROW_INSERT_AFTER_COMMAND) {
		doRowInsert(false);
	    } else if(command == ROW_DELETE_COMMAND) {
		doRowDelete();
	    } else if(command == COLUMN_INSERT_BEFORE_COMMAND) {
		doColumnInsert(true);
	    } else if(command == COLUMN_INSERT_AFTER_COMMAND) {
		doColumnInsert(false);
	    } else if(command == COLUMN_DELETE_COMMAND) {
		doColumnDelete();
	    } else if(command == MERGE_CELL_COMMAND) {
		doCellMerge();
	    } else if(command == SPLIT_CELL_COMMAND) {
		doCellSplit();
	    } else if(command == SORT_ASCENDING_COMMAND) {
		doSort(true);
	    } else if(command == SORT_DESCENDING_COMMAND) {
		doSort(false);
	    } else if(command == SET_COLUMN_NAME_COMMAND) {
		doSetColumnName();
	    } else if(command == MOVE_CELL_COMMAND) {
		doMove();		
	    } else if(command == CLEAR_CELL_COMMAND) {
		doClear();
	    } else if(command == FILL_CELL_COMMAND) {
		doFill();
	    } else if(command == TIP_COMMAND) {
		doTip();
	    } else if(command == ABOUT_COMMAND) {
		doAbout();
	    }
	}
	// window events
	public void windowOpened(WindowEvent e) {}
	public void windowClosing(WindowEvent e) {
	    doExit();
	}
	public void windowClosed(WindowEvent e) {}
	public void windowIconified(WindowEvent e) {}
	public void windowDeiconified(WindowEvent e) {}
	public void windowActivated(WindowEvent e) {}
	public void windowDeactivated(WindowEvent e) {}
	// mouse events
	public void mouseClicked(MouseEvent e) {}
	public void mousePressed(MouseEvent e) {
	    showPopup(e);
	}
	public void mouseReleased(MouseEvent e) {
	    showPopup(e);		
	}
	public void mouseEntered(MouseEvent e) {
	    String msg = "";	    
	    if(e.getSource() instanceof AbstractButton) {
		Action a = ((AbstractButton)e.getSource()).getAction();
		if(a != null) {
		    msg = (String)a.getValue(Action.LONG_DESCRIPTION);
		}  else {
		    msg = ((AbstractButton)e.getSource()).getToolTipText();
		}
	    } else if(e.getSource() instanceof JComponent) {
		msg = ((JComponent)e.getSource()).getToolTipText();
	    }
	    showStatus(msg);				
	}
	public void mouseExited(MouseEvent e) {
	    showStatus("");
	}
	private void showPopup(MouseEvent e) {
	    if(e.getSource() instanceof JDataGrid) {
		if(e.isPopupTrigger()) {
		    getPopupMenu().show((Component)e.getSource(), e.getX(), e.getY());
		}
	    }
	}
    }
    private void showStatus(String status) {
	// use create JStatusBar
	statusBar.setText(status);
    }
    private JPopupMenu getPopupMenu() {
	if(popupMenu == null) {
	    popupMenu = createPopupMenu();
	    addListeners(popupMenu);
	}
	return popupMenu;
    }
    private JPopupMenu createPopupMenu() {
	JPopupMenu popup = new JPopupMenu();
	popup.add(createMenuItem(UNDO_COMMAND));
	popup.add(createMenuItem(REDO_COMMAND));
	popup.addSeparator();
	popup.add(createMenuItem(CUT_COMMAND));
	popup.add(createMenuItem(COPY_COMMAND));
	popup.add(createMenuItem(PASTE_COMMAND));
	popup.addSeparator();	
	popup.add(createMenuItem(SELECT_ALL_COMMAND));		
	return popup;
    }
    private void addListeners(JComponent comp) {
	if(comp instanceof MenuElement) {
	    MenuElement[] mes = ((MenuElement)comp).getSubElements();
	    addListeners(mes);
	} else {
	    for(int i=0,n=comp.getComponentCount(); i<n; i++) {
		Component c = comp.getComponent(i);
		if(c instanceof AbstractButton || c instanceof JComboBox) {
		    c.addMouseListener(handler);
		}
	    }
	}
    }
    private void addListeners(MenuElement[] mes) {
	for(int i=0; i<mes.length; i++) {
	    if(mes[i] instanceof JMenuItem) {
		((JMenuItem)mes[i]).addMouseListener(handler);
	    }
	    addListeners(mes[i].getSubElements());
	}
    }
    public static void main(String[] args) {
	// fix L&F
	MetalLookAndFeel.setCurrentTheme(new FixedMetalTheme());	
	UIManager.getDefaults().put("Application.useSystemFontSettings", Boolean.TRUE);
	// end fix L&F
	Map map = new HashMap(1);
	// map.put(JCommonPane.BACKGROUND_IMAGE, getResource(getString("splash-image")));
	map.put(JCommonPane.PRODUCT_NAME, getString("product-name"));
	map.put(JCommonPane.COPYRIGHT_NOTE, getString("copyright-note"));	
	Map returnMap = JCommonPane.showSplashWindow(null, map);
	DataGridDemo demo = new DataGridDemo();
	final Frame f = demo.getFrame();
	f.pack();
	centerOnScreen(f);
	((Window)returnMap.get(JCommonPane.WINDOW)).hide();
	f.show();
	SwingUtilities.invokeLater(new Runnable() {
		public void run() {
		    showTipOfTheDayDialog(f);
		}
	    });
    }

    private static void showTipOfTheDayDialog(Frame f) {
	Map map = new HashMap(3);
	map.put(JCommonPane.TIP_URL, getResource(getString("tips-url")));
	JCommonPane.showTipOfTheDayDialog(f, map);
    }
    public static void resizeWindow(Window w) {
	Dimension d = w.getPreferredSize();
	Toolkit toolkit = w.getToolkit();
	Dimension size = toolkit.getScreenSize();	
	d.width = Math.min(d.width, size.width);
	d.height = Math.min(d.height, size.height);
	w.setSize(d);
	centerOnScreen(w);
	w.validate();	
    }
    
    //
    // move to core package in the future...
    //
    
    public static void centerOnScreen(Window window) {
	Dimension d = Toolkit.getDefaultToolkit().getScreenSize();
	window.setLocation((d.width - window.getWidth())/2,
			   (d.height - window.getHeight())/2);
    }
    private JMenu createMenu(String name) {
	JMenu menu = new JMenu();
	menu.setRequestFocusEnabled(false);	
	String label = getString(name+".name");
	Integer mnemonic = getMnemonic(name+".mnemonic");
	if(label != null) {
	    menu.setText(label);
	}
	if(mnemonic != null) {
	    menu.setMnemonic(mnemonic.intValue());
	}
	return menu;
    }
    // create component use actions
    private JMenuItem createMenuItem(String command) {
	JMenuItem item = new JMenuItem(getAction(command));
	item.setRequestFocusEnabled(false);
	if(item.getIcon() == null) {
	    item.setIcon(EmptyIcon.INSTANCE); // empty icon
	}
	// JDK1.3 cannot configure action correct, remove this when update to 1.4		
	item.setActionCommand(command);
	item.setAccelerator((KeyStroke)item.getAction().getValue(Action.ACCELERATOR_KEY));
	// end 
	return item;
    }
    private JButton createButton(String command) {
	JButton b = new JButton(getAction(command));
	b.setRequestFocusEnabled(false);	
	b.putClientProperty("hideActionText", Boolean.TRUE);
	// JDK1.3 cannot configure action correct, remove this when update to 1.4	
	b.setActionCommand(command); 
	b.setText("");
	// end
	return b;
    }
    private Action createAction(String command) {
	Action a = null;
	if(command != null) {
	    // action command and listener
	    a = new DefaultAction(command, handler);
	    // name
	    String name = getString(command+".name");
	    // mnemonic
	    Integer mnemonic = getMnemonic(command+".mnemonic");
	    // small icon
	    Icon icon = getIcon(command+".smicon");
	    // description
	    String desc = getString(command+".desc");
	    // accelerator key
	    KeyStroke key = getKeyStroke(command+".accel");
	    if(name != null) {
		a.putValue(Action.NAME, name);
	    }
	    if(mnemonic != null) {
		a.putValue(Action.MNEMONIC_KEY, mnemonic);
	    }
	    if(icon != null) {
		a.putValue(Action.SMALL_ICON, icon);
	    }
	    if(desc != null) {
		a.putValue(Action.SHORT_DESCRIPTION, desc);
		a.putValue(Action.LONG_DESCRIPTION, desc);		
	    }
	    if(key != null) {
		a.putValue(Action.ACCELERATOR_KEY, key);
	    }
	}
	return a;
    }
    private Action getAction(String command) {
	Action a = (Action)actionMap.get(command);
	if(a == null) {
	    a = createAction(command);
	    actionMap.put(command, a);
	}
	return a;
    }
    // load action support    
    private static Integer getMnemonic(String name) {
	String value = getString(name);
	if(value != null && value.length() > 0) {
	    return new Integer(value.charAt(0));
	}
	return null;
    }
    private static Icon getIcon(String name) {
	String value = getString(name);
	if(value != null) {
	    return new LazyImageIcon(getResource(value));
	}
	return null;
    }
    private static KeyStroke getKeyStroke(String name) {
	String value = getString(name);
	if(value != null) {
	    return KeyStroke.getKeyStroke(value);
	}
	return null;
    }
    private static String getString(String name) {
	String value = null;
	try {
	    value = bundle.getString(name);
	} catch (MissingResourceException e) {
	    System.err.println(e);
	}
	if("".equals(value)) {
	    return null;
	} else {
	    return value;
	}
    }
    private static URL getResource(String name) {
	URL url = DataGridDemo.class.getResource(name);
	return url;
    }
    // Todo: should use reflect or delegate to some object?
    private static class DefaultAction extends AbstractAction {
	private ActionListener listener;
	public DefaultAction(String actionCommand, ActionListener actionListener) {
	    super();
	    putValue(ACTION_COMMAND_KEY, actionCommand);
	    this.listener = actionListener;
	}
	public void actionPerformed(ActionEvent e) {
	    listener.actionPerformed(e);
	}
    }
    private static class LazyImageIcon extends ImageIcon {
        private transient URL location;
	private transient Image img;
        public LazyImageIcon(URL location) {
            super();
            this.location = location;
	    // loading image in background thread
	    img = Toolkit.getDefaultToolkit().getImage(location);
        }
        public void paintIcon(Component c, Graphics g, int x, int y) {
            if (getImage() != null) {
                super.paintIcon(c, g, x, y);
            }
        }
        public int getIconWidth() {
            if (getImage() != null) {
                return super.getIconWidth();
            }
            return 0;
        }
        public int getIconHeight() {
            if (getImage() != null) {
                return super.getIconHeight();
            }
            return 0;
        }
        public Image getImage() {
            if (location != null) {
                setImage(img);
                location = null;
            }
            return super.getImage();
        }	
    }
    private static class EmptyIcon implements Icon {
	static final Icon INSTANCE = new EmptyIcon();
	private EmptyIcon() {}
	public void paintIcon(Component c, Graphics g, int x, int y) {
	}
	public int getIconWidth() {
	    return 16;
	}
	public int getIconHeight() {
	    return 16;
	}
    }
    private static class FixedMetalTheme extends DefaultMetalTheme {
	private FontUIResource font0;
	private FontUIResource font3;
	private FontUIResource font4;
	public String getName() { return "Fixed"; }
	public FontUIResource getControlTextFont() {
	    FontUIResource f = super.getControlTextFont();
	    if(f.isBold()) {
		if(font0 == null) {
		    font0 = new FontUIResource(f.deriveFont(Font.PLAIN));
		}
		return font0;
	    }
	    return f;	    
	}
	// public FontUIResource getSystemTextFont();
	// public FontUIResource getUserTextFont();
	public FontUIResource getMenuTextFont() {
	    FontUIResource f = super.getControlTextFont();
	    if(f.isBold()) {
		if(font3 == null) {
		    font3 = new FontUIResource(f.deriveFont(Font.PLAIN));
		}
		return font3;
	    }
	    return f;	    
	}
	public FontUIResource getWindowTitleFont() {
	    FontUIResource f = super.getControlTextFont();
	    if(f.isBold()) {
		if(font4 == null) {
		    font4 = new FontUIResource(f.deriveFont(Font.PLAIN));
		}
		return font4;
	    }
	    return f;
	}
	// public FontUIResource getSubTextFont();
    }
}






