/*
 * $Id: DemoPanel.java,v 1.27 2003/11/15 00:54:07 zfq Exp $
 *
 * Copyright (C) 2001-2003 Extreme Component, Inc. All rights reserved.
 * Use is subject to license terms. 
 */


import java.util.EventListener;

import java.io.Serializable;
import java.io.ObjectOutputStream;
import java.io.ObjectInputStream;
import java.io.IOException;
import java.io.*;
import java.net.*;

import java.util.*;

import java.awt.*;
import java.awt.event.*;
import java.beans.*;

import javax.swing.*;
import javax.swing.event.*;
import javax.swing.border.*;
import javax.swing.plaf.*;
import javax.swing.text.*;
import javax.swing.text.html.*;

import javax.accessibility.*;

/**
 * DemoPanel
 *
 * @author $Author: zfq $
 * @version $Revision: 1.27 $ $Date: 2003/11/15 00:54:07 $
 */
public class DemoPanel extends JPanel {

    private String COPYRIGHT_MESSAGE = "Copyright (C) 2001-2003 Extreme Component, Inc.\n" + 
	                               "All rights reserved.";
    private JToolBar toolBar;
    private JTabbedPane tabbedPane;
    private JPanel sourceCodePanel;
    private JPanel javaDocPanel;
    private String demoName;

    
    public DemoPanel() {
	setLayout(new BorderLayout());
	initComponents();	
	add(createMenuBar(), BorderLayout.NORTH);
	add(createCenterPanel(), BorderLayout.CENTER);
    }

    public void addDemo(String name, JComponent c) {
    }
    
    public void addDemo(JComponent c) {
	tabbedPane.removeAll();
	demoName = getDemoName(c);
	loaded = false;
	tabbedPane.addTab(demoName, c);
	if(isSourceCodeAvaliable()) {
	    tabbedPane.addTab("Source Code", getSourceCodePanel());
	}
	if(isJavaDocAvaliable()) {
	    tabbedPane.addTab("JavaDoc", getJavaDocPanel());
	}
	tabbedPane.setSelectedIndex(0);	
    }

    private String getDemoName(JComponent c) {
	String className = c.getClass().getName();
	// debug
	// System.out.println("demo class name: " + className);
	return className;
    }

    private void initComponents() {
	// start init create source panel.
	sourceCodePanel = createSourceCodePanel();
	codeMap = new HashMap();
	// end init
	javaDocPanel = createJavaDocPanel();
    }
    
    // source code panel
    private JTextArea textArea;    
    private JPanel createSourceCodePanel() {
	JPanel panel = new JPanel();
	panel.setLayout(new BorderLayout());
	textArea = new JTextArea(20, 50);
	textArea.setFont(new Font("Monospaced", Font.PLAIN, 12));
	panel.add(new JScrollPane(textArea), BorderLayout.CENTER);
	return panel;
    }

    // java doc panel
    private JEditorPane editorPane;
    private JPanel createJavaDocPanel() {
	JPanel panel = new JPanel();
	editorPane = new JEditorPane();
	editorPane.setEditable(false);
	editorPane.addHyperlinkListener(createHyperLinkListener());	
	panel.setLayout(new BorderLayout());
	panel.add(new JScrollPane(editorPane), BorderLayout.CENTER);
	return panel;
    }

    private void showWaitCursor(Component c) {
	c.setCursor(Cursor.getPredefinedCursor(Cursor.WAIT_CURSOR));
    }

    private void showDefaultCursor(Component c) {
        c.setCursor(Cursor.getDefaultCursor());	    
    }
    
    private HyperlinkListener createHyperLinkListener() {
	return new HyperlinkListener() {
	    public void hyperlinkUpdate(HyperlinkEvent e) {
		showWaitCursor(editorPane);
		if (e.getEventType() == HyperlinkEvent.EventType.ACTIVATED) {
		    if (e instanceof HTMLFrameHyperlinkEvent) {
			((HTMLDocument)editorPane.getDocument()).processHTMLFrameHyperlinkEvent(
			    (HTMLFrameHyperlinkEvent)e);
		    } else {
			try {
			    editorPane.setPage(e.getURL());
			} catch (IOException ioe) {
			    System.out.println("IOE: " + ioe);
			}
		    }
		}
		showDefaultCursor(editorPane);
	    }
	};
    }
    
    private JPanel getSourceCodePanel() {
	return sourceCodePanel;
    }

    private JPanel getJavaDocPanel() {
	return javaDocPanel;
    }
    
    // center panel
    private JPanel createCenterPanel() {
	JPanel panel = new JPanel();
	panel.setLayout(new BorderLayout());
	toolBar = new JToolBar();
	toolBar.putClientProperty("JToolBar.isRollover", Boolean.TRUE);
	panel.add(toolBar, BorderLayout.NORTH);
	tabbedPane = new JTabbedPane();
	tabbedPane.getModel().addChangeListener(new TabListener());
	panel.add(tabbedPane, BorderLayout.CENTER);
	return panel;
    }

    
    private class TabListener implements ChangeListener {
	public void stateChanged(ChangeEvent e) {
	    SingleSelectionModel model = (SingleSelectionModel) e.getSource();
	    boolean srcSelected = model.getSelectedIndex() == 1;
	    boolean docSelected = model.getSelectedIndex() == 2;
	    if(srcSelected) {
		showSourceCode();
	    } else if(docSelected) {
		showJavaDoc();
	    } else {
		// do nothing
	    }
	}
    }

    private boolean isJavaDocAvaliable() {
	URL url = getJavaDocURL(demoName);
	return url != null;
    }

    private boolean isSourceCodeAvaliable() {
	return getSourceCodeURL(demoName) != null;
    }
    
    private URL getJavaDocURL(String demoName) {
	String fileName = "api/index.html";
	// return getClass().getClassLoader().getSystemResource(fileName);
	return getClass().getResource(fileName);	    
    }

    private URL getSourceCodeURL(String demoName) {
	if(demoName == null)
	    return null;
	String fileName = "src/" + demoName + ".java";
	// System.out.println("source code file: " + fileName);	
	return getClass().getResource(fileName);
    }

    private boolean loaded;
    private void showSourceCode() {
	final URL url = getSourceCodeURL(demoName);
	// System.out.println("source code: " + url);
	if(url != null) {
	    if(!codeMap.containsKey(url)) {
		// first loading
		textArea.setText("Source Code loading...");
		textArea.repaint();

		Runnable runner = new Runnable() {
			public void run() {
			    String source = loadSourceCode(url);			    
			    textArea.setText(source);
			    textArea.setCaretPosition(0);
			    loaded = true;
			}
		    };
		SwingUtilities.invokeLater(runner);		
	    } else {
		if(!loaded) {
		    String source = (String)codeMap.get(url);
		    textArea.setText(source);
		    textArea.setCaretPosition(0);
		    loaded = true;		    
		}
	    }
	}
    }

    private void showJavaDoc() {
	final URL url = getJavaDocURL(demoName);
	// System.out.println("java doc: " + url);
	Runnable runner = new Runnable() {
		public void run() {
		    try {
			editorPane.setPage(url);
		    } catch (Exception e) {
			System.out.println("Could not load file: " + url + "\nError: " + e.getMessage());
		    }
		}
	    };
	SwingUtilities.invokeLater(runner);			

    }
    
    private Map codeMap;    

    private String loadSourceCode(URL url) {
	String sourceString = null;
	StringBuffer sourceCode = new StringBuffer();
	InputStream is;
	InputStreamReader isr;	
	try {
	    is = url.openStream();
	    isr = new InputStreamReader(is);
	    BufferedReader reader = new BufferedReader(isr);
	    char[] buf = new char[8192];
	    while(true) {
		int i = reader.read(buf);
		if(i != -1) {
		    sourceCode.append(new String(buf, 0, i));
		} else {
		    break;
		}
	    }
	    sourceString = sourceCode.toString();
	    codeMap.put(url, sourceString);
	    return sourceString;
	} catch (Exception e) {
	    sourceCode.append("Could not load file: " + url + "\nError: " + e.getMessage());
	}
	return sourceCode.toString();
    }
    
	
    private JMenuBar createMenuBar() {
	JPopupMenu.setDefaultLightWeightPopupEnabled(false);
	JMenuBar menuBar = new JMenuBar();
	menuBar.add(createFileMenu());
	menuBar.add(createLookAndFeelMenu());
	//menuBar.add(createLocaleMenu());
	menuBar.add(createOrientationMenu());
	menuBar.add(createHelpMenu());
	return menuBar;
    }

    private JMenu createFileMenu() {
	JMenu menu = new JMenu("File");
	menu.setMnemonic('F');
	JMenuItem exitItem = new JMenuItem("Exit");
	exitItem.setMnemonic('x');
	exitItem.addActionListener(new ActionListener() {
		public void actionPerformed(ActionEvent e) {
		    System.exit(0);
		}
	    });
	menu.add(exitItem);
	return menu;
    }
    
    private JMenu createLookAndFeelMenu() {
	JMenu menu = new JMenu("Look&Feel");
	menu.setMnemonic('L');
	UIManager.LookAndFeelInfo[] infos = UIManager.getInstalledLookAndFeels();
	ButtonGroup group = new ButtonGroup();
	String currentName = UIManager.getLookAndFeel().getName();
	ActionListener listener = new LookAndFeelActionListener();
	for(int i=0, count=infos.length; i<count; i++) {
	    JRadioButtonMenuItem rbItem = new JRadioButtonMenuItem(infos[i].getName());
	    if(currentName.equals(infos[i].getName())) {
		rbItem.setSelected(true);
	    }
	    rbItem.setActionCommand(infos[i].getClassName());
	    rbItem.addActionListener(listener);
	    group.add(rbItem);
	    menu.add(rbItem);
	    
	}
	return menu;
    }

    private class LookAndFeelActionListener implements ActionListener {
	public void actionPerformed(ActionEvent e) {
	    String command = e.getActionCommand();
	    try {
		UIManager.setLookAndFeel(command);
	    } catch (Exception exce) {
		JOptionPane.showMessageDialog(getWindow(), exce.getMessage());
		return;
	    }
	    SwingUtilities.updateComponentTreeUI(getWindow());
	    resizeWindow(getWindow());
	}
    }

    private HashMap localMenuMap = new HashMap();
    
    private JMenu createLocaleMenu() {
	JMenu menu = new JMenu("Locale");
	menu.setMnemonic('c');
	Locale[] locales = Locale.getAvailableLocales();
	Locale defaultLocale = Locale.getDefault();
	ActionListener listener = new LocaleActionListener();
	ButtonGroup group = new ButtonGroup();
	for(int i=0,count=locales.length; i<count; i++) {
	    JRadioButtonMenuItem rbItem = new JRadioButtonMenuItem(locales[i].getDisplayName());
	    if(locales[i].equals(defaultLocale)) {
		rbItem.setSelected(true);
	    }
	    localMenuMap.put(rbItem, locales[i]);
	    rbItem.addActionListener(listener);
	    group.add(rbItem);
	    menu.add(rbItem);	    
	}
	return menu;
    }

    private class LocaleActionListener implements ActionListener {
	public void actionPerformed(ActionEvent e) {
	    JRadioButtonMenuItem menu = (JRadioButtonMenuItem)e.getSource();
	    Locale locale = (Locale)localMenuMap.get(menu);
	    if(locale != null) {
		Locale.setDefault(locale);
		getWindow().setLocale(locale);
		SwingUtilities.updateComponentTreeUI(getWindow());
		resizeWindow(getWindow());		
	    }
	}
    }
    
    private JMenu createOrientationMenu() {
	JMenu menu = new JMenu("Orientation");
	menu.setMnemonic('O');
	ActionListener orientationListener = new OrientationActionListener();
	
	ButtonGroup group = new ButtonGroup();
	JRadioButtonMenuItem rbItem = new JRadioButtonMenuItem("Left To Right");
	rbItem.setMnemonic('L');
	rbItem.setActionCommand("lefttoright");
	rbItem.addActionListener(orientationListener);
	group.add(rbItem);
	menu.add(rbItem);

	rbItem =  new JRadioButtonMenuItem("Right To Left");
	rbItem.setMnemonic('R');	
	rbItem.setActionCommand("righttoleft");
	rbItem.addActionListener(orientationListener);	
	group.add(rbItem);
	menu.add(rbItem);

	rbItem =  new JRadioButtonMenuItem("Unknown");
	rbItem.setMnemonic('U');	
	rbItem.setSelected(true);
	rbItem.setActionCommand("unknown");
	rbItem.addActionListener(orientationListener);	
	group.add(rbItem);
	menu.add(rbItem);
	
	return menu;
    }

    private JMenu createHelpMenu() {
	JMenu menu = new JMenu("Help");
	menu.setMnemonic('H');
	JMenuItem aboutItem = new JMenuItem("About");
	aboutItem.setMnemonic('A');
	aboutItem.addActionListener(new ActionListener() {
		public void actionPerformed(ActionEvent e) {
		    // showDialog
		    JOptionPane.showMessageDialog(getWindow(), COPYRIGHT_MESSAGE, 
						  "About", JOptionPane.INFORMATION_MESSAGE);
		}
	    });
	menu.add(aboutItem);
	return menu;
				    
    }
    
    private Window getWindow() {
	for(Container c = this; c != null; c = c.getParent()) {
	    if(c instanceof Window) {
		return (Window)c;
	    }
	}
	return null;
    }
    
    private class OrientationActionListener implements ActionListener {

	public void actionPerformed(ActionEvent e) {
	    String command = e.getActionCommand();
	    Window w = getWindow();
	    ComponentOrientation orientation = null;
	    if(command.equals("lefttoright")) {
		orientation = ComponentOrientation.LEFT_TO_RIGHT;
	    } else if(command.equals("righttoleft")) {
		orientation = ComponentOrientation.RIGHT_TO_LEFT;
	    } else if(command.equals("unknown")) {
		orientation = ComponentOrientation.UNKNOWN;
	    }
	    setComponentTree(w, orientation);
	    resizeWindow(w);	    
	}
						    
    }

    private void setComponentTree(Component c, ComponentOrientation orientation) {
	if(c instanceof Container) {
	    Component[] components = ((Container)c).getComponents();
	    for(int i=0; i<components.length; i++) {
		components[i].setComponentOrientation(orientation);
		components[i].invalidate();
		setComponentTree(components[i], orientation);
	    }
	}
    }

    private void resizeWindow(Window w) {
	Dimension d = w.getPreferredSize();
	Toolkit toolkit = w.getToolkit();
	Dimension size = toolkit.getScreenSize();	
	d.width = Math.min(d.width, size.width);
	d.height = Math.min(d.height, size.height);
	w.setSize(d);
	centerOnScreen(w);
	w.validate();	
    }

    public static void centerOnScreen(Window window) {
	Dimension d = Toolkit.getDefaultToolkit().getScreenSize();
	window.setLocation((d.width - window.getWidth())/2,
		    (d.height - window.getHeight())/2);
    }    

    // Todo:
    // add toolbar, add beaninfo images ... 32x32
    // add Source panel, JavaDoc pannel, ComponentDemo panel

}
